@@ -548,15 +548,48 @@ GSM_Error ATGEN_DecodePDUMessage(GSM_StateMachine *s, const char *PDU, const int
 	return ERR_NONE;
 }
 
+GSM_Error ATGEN_ReadSMSText(GSM_Protocol_Message *msg, GSM_StateMachine *s, GSM_SMSMessage *sms)
+{
+	int i;
+	const char *line;
+	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
+	size_t length = 0;
+	GSM_Error error;
+
+	/* Go trough all lines till OK */
+	for (i = 3; strcmp(line = GetLineString(msg->Buffer, &Priv->Lines, i), "OK") != 0; i++) {
+		if (i > 3) {
+			/* Include new line */
+			sms->Text[(2 * sms->Length) + 0] = 0;
+			sms->Text[(2 * sms->Length) + 1] = '\n';
+			sms->Text[(2 * sms->Length) + 2] = 0;
+			sms->Text[(2 * sms->Length) + 3] = 0;
+			sms->Length++;
+		}
+		length = GetLineLength(msg->Buffer, &Priv->Lines, i);
+		error = ATGEN_DecodeText(s, line, length,
+			sms->Text + (2 * sms->Length),
+			sizeof(sms->Text) - (2 * sms->Length),
+			TRUE, FALSE);
+		if (error != ERR_NONE) {
+			return error;
+		}
+		sms->Length += length;
+	}
+	return ERR_NONE;
+}
+
 GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *s)
 {
 	GSM_Error error = ERR_UNKNOWN;
 	GSM_Phone_ATGENData *Priv = &s->Phone.Data.Priv.ATGEN;
 	GSM_SMSMessage *sms = &s->Phone.Data.GetSMSMessage->SMS[0];
-	unsigned char buffer[300] = {'\0'}, firstbyte = 0, TPDCS = 0, TPUDL = 0, TPStatus = 0;
-	const char *line;
-	size_t length = 0;
+	unsigned char buffer[3000] = {'\0'}, firstbyte = 0, TPDCS = 0, TPUDL = 0, TPStatus = 0, TPPID = 0;
 	int current = 0, i = 0;
+	int state;
+	unsigned char *ptr;
+	char *comma;
+	char *expected_comma;
 
 	switch (Priv->ReplyState) {
 	case AT_Reply_OK:
@@ -569,25 +602,60 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 		case SMS_AT_PDU:
 			CopyLineString(buffer, msg->Buffer, &Priv->Lines, 2);
 
-			/* Siemens MC35 (only ?) */
-			if (strstr(msg->Buffer,"+CMGR: 0,,0")!=NULL) {
-				return ERR_EMPTY;
-			}
-			/* @bug Hardcoded position is wrong! */
-			return ATGEN_DecodePDUMessage(s,
-					GetLineString(msg->Buffer,&Priv->Lines,3),
-					atoi(buffer + 7));
-		case SMS_AT_TXT:
-			GSM_SetDefaultReceivedSMSData(sms);
-			error = ATGEN_ParseReply(s,
-					GetLineString(msg->Buffer, &Priv->Lines, 2),
-					"+CMGR: @r, @p, @d",
-					buffer, sizeof(buffer), sms->Number, sizeof(sms->Number),
-					&sms->DateTime);
-
+			/* Parse reply */
+			error = ATGEN_ParseReply(s, buffer, "+CMGR: @i, @0", &state);
 			if (error != ERR_NONE) {
 				return error;
 			}
+
+			/* Siemens MC35 (only ?) */
+			if (strcmp(buffer, "+CMGR: 0,,0") == 0) {
+				return ERR_EMPTY;
+			}
+
+			error = ATGEN_DecodePDUMessage(s, GetLineString(msg->Buffer,&Priv->Lines,3), state);
+			return error;
+		case SMS_AT_TXT:
+			GSM_SetDefaultReceivedSMSData(sms);
+
+			/*
+			 * This is just a hack until proper parsing of text mode is done.
+			 * It uses old style of manual parsing, to skip entries parsed above.
+			 */
+			current = 0;
+
+			/* Skip to first : */
+			while (msg->Buffer[current] != ':') {
+				current++;
+			}
+			current++;
+			/* Skip any spaces */
+			while (msg->Buffer[current] == ' ') {
+				current++;
+			}
+
+			/* Grab first parameter */
+			current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+
+			/* Remove leading " */
+			for (ptr = buffer; *ptr == '"'; ptr++);
+			if (ptr != buffer) {
+				memmove (buffer, ptr, strlen (ptr) + 1);
+			}
+
+			/* Go to the end of string */
+			for (ptr = buffer; *ptr; ptr++);
+			/* Remove trailing " */
+			ptr--;
+			while (ptr >= buffer && *ptr == '"') {
+				ptr--;
+			}
+			ptr++;
+			*ptr = 0;
+
+			smprintf(s, "Message type: %s\n", buffer);
+
+			/* Check message type */
 			if (!strcmp(buffer,"0") || !strcmp(buffer,"REC UNREAD")) {
 				smprintf(s, "SMS type - deliver\n");
 				sms->State 	 = SMS_UnRead;
@@ -632,7 +700,12 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 					sms->Folder = 4; /*OUTBOX ME*/
 				}
 				sms->InboxFolder = FALSE;
+			} else {
+				smprintf(s, "Uknown message state: %s\n", buffer);
+				return ERR_UNKNOWN;
 			}
+
+			/* Do we have detailed format? */
 			if (Priv->SMSTextDetails == FALSE) {
 				sms->Class = 1;
 				sms->Coding = SMS_Coding_Default_No_Compression;
@@ -642,61 +715,25 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 				sms->SMSC.Number[1]=0;
 				sms->ReplyViaSameSMSC = FALSE;
 
-				/* Go trough all lines till OK */
-				for (i = 3; strcmp(line = GetLineString(msg->Buffer, &Priv->Lines, i), "OK") != 0; i++) {
-					if (i > 3) {
-						/* Include new line */
-						sms->Text[(2 * sms->Length) + 0] = 0;
-						sms->Text[(2 * sms->Length) + 1] = '\n';
-						sms->Text[(2 * sms->Length) + 2] = 0;
-						sms->Text[(2 * sms->Length) + 3] = 0;
-						sms->Length++;
-					}
-					length = GetLineLength(msg->Buffer, &Priv->Lines, i);
-					error = ATGEN_DecodeText(s, line, length,
-						sms->Text + (2 * sms->Length),
-						sizeof(sms->Text) - (2 * sms->Length),
-						TRUE, FALSE);
-					if (error != ERR_NONE)
-						return error;
-					sms->Length += length;
-				}
-				return ERR_NONE;
+				return ATGEN_ReadSMSText(msg, s, sms);
 			}
 
-			/*
-			 * This is just a hack until proper parsing of text mode is done.
-			 * It uses old style of manual parsing, to skip entries parsed above.
-			 */
-			current = 0;
-
-			while (msg->Buffer[current]!='"') {
-				current++;
-			}
 			current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-			current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-
 			/* It's delivery report according to Nokia AT standards */
-			if (sms->Folder == 1 && buffer[0]!=0 && buffer[0]!='"') {
+			if ((sms->Folder == 1 || sms->Folder == 3) && buffer[0]!=0 && buffer[0]!='"') {
 				/* ??? */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
-				/* format of sender number */
+				/* Sender number */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
-				/* Sender number */
 				/* FIXME: support for all formats */
 				EncodeUnicode(sms->Number,buffer+1,strlen(buffer)-2);
 				smprintf(s, "Sender \"%s\"\n",DecodeUnicodeString(sms->Number));
 
-				/* ??? */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+				firstbyte = atoi(buffer);
 
-				/* Sending datetime */
-				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-				i = strlen(buffer);
-				buffer[i] = ',';
-				i++;
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer+i);
 				smprintf(s, "\"%s\"\n",buffer);
 				error = ATGEN_DecodeDateTime(s, &sms->DateTime, buffer);
@@ -719,6 +756,7 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 				/* TPStatus */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				TPStatus = atoi(buffer);
+				buffer[PHONE_SMSDeliver.firstbyte]     = firstbyte;
 				buffer[PHONE_SMSStatusReport.TPStatus] = TPStatus;
 				error = GSM_DecodeSMSFrameStatusReportData(&(s->di), sms, buffer, PHONE_SMSStatusReport);
 
@@ -731,37 +769,28 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 				sms->PDU = SMS_Status_Report;
 				sms->ReplyViaSameSMSC = FALSE;
 			} else {
-				/* Sender number */
 				/* FIXME: support for all formats */
 				EncodeUnicode(sms->Number,buffer+1,strlen(buffer)-2);
 
-				/* Sender number in alphanumeric format ? */
-				current += ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-
 				if (strlen(buffer)!=0) {
 					EncodeUnicode(sms->Number,buffer+1,strlen(buffer)-2);
 				}
 				smprintf(s, "Sender \"%s\"\n",DecodeUnicodeString(sms->Number));
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* Sending datetime */
+
 				if (sms->Folder == 1 || sms->Folder == 3) {
 					current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-					/* FIXME: ATGEN_ExtractOneParameter() is broken as it doesn't respect
-					 * quoting of parameters and thus +FOO: "ab","cd,ef" will consider
-					 * as three arguments: "ab" >> "cd >> ef"
-					 */
-					if (*buffer=='"') {
-						i = strlen(buffer);
-						buffer[i] = ',';
-						i++;
-						current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer+i);
-					}
+
 					smprintf(s, "\"%s\"\n",buffer);
 
-					if (*buffer)
+					if (*buffer) {
 						error = ATGEN_DecodeDateTime(s, &sms->DateTime, buffer);
-						if (error != ERR_NONE) return error;
-					else {
+						if (error != ERR_NONE) {
+							return error;
+						}
+					} else {
 						/* FIXME: What is the proper undefined GSM_DateTime ? */
 						memset(&sms->DateTime, 0, sizeof(sms->DateTime));
 					}
@@ -771,39 +800,46 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 						return error;
 					}
 				}
-				/* Sender number format */
+
+				/* address type */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 
 				/* First byte */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
 				firstbyte = atoi(buffer);
 				sms->ReplyViaSameSMSC = FALSE;
+				smprintf (s, "buffer firstbyte:%s\n", buffer);
+
+				/* TP PID */
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+				sms->ReplaceMessage = 0;
+
+				TPPID = atoi(buffer);
+
+				/* TP DCS */
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+				TPDCS = atoi(buffer);
+				smprintf(s, "TPDCS: %02x\n", TPDCS);
+				/* SMSC number */
+				/* FIXME: support for all formats */
+				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+
+				if (buffer[0] != '"' && buffer[0]) {
+					/*TP VP */
+					current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
+				}
+
+				EncodeUnicode(sms->SMSC.Number,buffer+1,strlen(buffer)-2);
 
 				/* GSM 03.40 section 9.2.3.17 (TP-Reply-Path) */
 				if ((firstbyte & 128)==128) {
 					sms->ReplyViaSameSMSC = TRUE;
 				}
-				/* TP PID */
-				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-				sms->ReplaceMessage = 0;
 
-				if (atoi(buffer) > 0x40 && atoi(buffer) < 0x48) {
-					sms->ReplaceMessage = atoi(buffer) - 0x40;
+				if (TPPID > 0x40 && TPPID < 0x48) {
+					sms->ReplaceMessage = TPPID - 0x40;
 				}
-				smprintf(s, "TPPID: %02x %i\n",atoi(buffer),atoi(buffer));
-
-				/* TP DCS */
-				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-				TPDCS = atoi(buffer);
-
-				if (sms->Folder == 2 || sms->Folder == 4) {
-					/*TP VP */
-					current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-				}
-				/* SMSC number */
-				/* FIXME: support for all formats */
-				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
-				EncodeUnicode(sms->SMSC.Number,buffer+1,strlen(buffer)-2);
+				smprintf(s, "TPPID: %02x %i\n", TPPID, TPPID);
 
 				/* Format of SMSC number */
 				current+=ATGEN_ExtractOneParameter(msg->Buffer+current, buffer);
@@ -814,22 +850,11 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 				current++;
 				sms->Coding = GSM_GetMessageCoding(&(s->di), TPDCS);
 				sms->Class = -1;
-
-				if ((TPDCS & 0xD0) == 0x10) {
-					/* bits 7..4 set to 00x1 */
-					if ((TPDCS & 0xC) == 0xC) {
-						smprintf(s, "WARNING: reserved alphabet value in TPDCS\n");
-					} else {
-						sms->Class = (TPDCS & 3);
-					}
-				} else if ((TPDCS & 0xF0) == 0xF0) {
-					/* bits 7..4 set to 1111 */
-					if ((TPDCS & 8) == 8) {
-						smprintf(s, "WARNING: set reserved bit 3 in TPDCS\n");
-					} else {
-						sms->Class = (TPDCS & 3);
-					}
+				/* GSM 03.40 section 9.2.3.10 (TP-Data-Coding-Scheme) and GSM 03.38 section 4 */
+				if ((TPDCS & 0xD0) == 0x10 || (TPDCS & 0xF0) == 0xF0) {
+					sms->Class = TPDCS & 3;
 				}
+
 				smprintf(s, "SMS class: %i\n",sms->Class);
 
 				switch (sms->Coding) {
@@ -839,14 +864,51 @@ GSM_Error ATGEN_ReplyGetSMSMessage(GSM_Protocol_Message *msg, GSM_StateMachine *
 					/* If UDH available, treat it as Unicode or 8 bit */
 					if ((firstbyte & 0x40)!=0x40) {
 						sms->UDH.Type	= UDH_NoUDH;
-						sms->Length	= TPUDL;
-						EncodeUnicode(sms->Text,msg->Buffer+Priv->Lines.numbers[2*2],TPUDL);
+						error = ATGEN_ReadSMSText(msg, s, sms);
+						if (sms->Length	== TPUDL + 4) {
+							char *tail;
+							tail = sms->Text + 2 * (UnicodeLength (sms->Text) - 4);
+							if (tail[0] == 0 && tail[1] == ',' && tail[4] == 0 && tail[5] == ',') {
+								tail[1] = 0;
+								sms->Length = TPUDL;
+							}
+						}
+						if (sms->Length	!= TPUDL) {
+							smprintf(s, "WARNING: Indicated message length (%d) does not match real (%d)\n", TPUDL, sms->Length);
+						}
 						break;
 					}
 				case SMS_Coding_Unicode_No_Compression:
 				case SMS_Coding_8bit:
+					if ((firstbyte & 0x40)==0x40 && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SMS_UTF8_ENCODED)) {
+						comma = strchr (msg->Buffer+current, ',');
+
+						if (sms->Coding == SMS_Coding_Default_No_Compression) {
+							expected_comma = (char *)msg->Buffer+current + ((7 * TPUDL + 7) / 8) * 2;
+						} else {
+							expected_comma = (char *)msg->Buffer+current + TPUDL * 2;
+						}
+						if (comma == expected_comma || !comma) {
+							comma = expected_comma;
+						} else {
+							smprintf (s, "UDL fix: %d,", TPUDL);
+							if (sms->Coding == SMS_Coding_Default_No_Compression) {
+								TPUDL = ((comma - ((char *)msg->Buffer+current)) * 4) / 7;
+							} else {
+								TPUDL = (comma - ((char *)msg->Buffer+current)) / 2;
+							}
+							smprintf (s, "%d\n", TPUDL);
+						}
+						DecodeHexBin(buffer+PHONE_SMSDeliver.Text, msg->Buffer+current, comma - (char *) (msg->Buffer+current));
+						buffer[PHONE_SMSDeliver.firstbyte] 	= firstbyte;
+						buffer[PHONE_SMSDeliver.TPDCS] 		= TPDCS;
+						buffer[PHONE_SMSDeliver.TPUDL] 		= TPUDL;
+						return GSM_DecodeSMSFrameText(&(s->di), sms, buffer, PHONE_SMSDeliver);
+					}
+
 					if (sms->Coding == SMS_Coding_Unicode_No_Compression && GSM_IsPhoneFeatureAvailable(s->Phone.Data.ModelInfo, F_SMS_UTF8_ENCODED)) {
 						DecodeUTF8(buffer+PHONE_SMSDeliver.Text, msg->Buffer+current, TPUDL);
+						TPUDL = 2 * UnicodeLength (buffer+PHONE_SMSDeliver.Text);
 					} else {
 						DecodeHexBin(buffer+PHONE_SMSDeliver.Text, msg->Buffer+current, TPUDL*2);
 					}
